#include "DEM.h"
#define MULT 2.5

DEM::DEM(char *aName)
{
  strcpy(name, aName);
};

DEM::DEM(ELEV **aData, int aDim): dim(aDim)
{
  allocMem();
  
  for(int i=0; i<dim; i++)
    for(int j=0; j<dim; j++)
      data[i][j] = aData[i][j];
};

void 
DEM::setData(ELEV **newData)
{
  DeleteMatrix(data);
  data = newData;
}

void
DEM::allocMem()
{
  NewMatrix(data, dim, dim, ELEV);
}

void 
DEM::freeMem()
{
  DeleteMatrix(data);
}

void
DEM::writeRect(int xfrom, int xto, int yfrom, int yto, char *fname)
{
  info("Writing the rectangle to a file...");
  info(fname);
  
  ofstream out(fname);
  
  out.put(0);
  out.put(0);
  out.put((xto-xfrom)/256);
  out.put((xto-xfrom)%256);
  out.put(0);
  out.put(0);
  out.put((yto-yfrom)/256);
  out.put((yto-yfrom)%256);

  for(int i=yfrom; i<yto; i++)
    for(int j=xto-1; j>=xfrom; j--) {
      out.put(int(data[i][j])/256);
      out.put(int(data[i][j])%256);
    }

  out.close();
  info("...done\n");
  
}

////////////////////////////////////////

void 
PGMDEM::read()
{
  int n, i, j;
  char buf[255];
  char c;

  FILE *f = fopen(name, "rb");
  if (f==NULL) fatal("Cannot open the data file!");
  // the pgm header -- P5 or sth
  fscanf(f, "%s", &buf);
  if ((c=fgetc(f))=='#') { //comment
    for(; (c=fgetc(f))!='\n'; ) ;
  } else ungetc(c, f);
  // the dimensions: number of columns and the rows, respectively
  fscanf(f, "%d %d", &dim, &dim);
  if ((c=fgetc(f))=='#') { //comment
    for(; (c=fgetc(f))!='\n'; ) ;
  } else ungetc(c, f);
  // the maximum value -- determines the data element size
  fscanf(f, "%d", &n);
  if (n!=255) 
    fatal("Can only handle 255-shades files");
  // and the last carridge return character
  fgetc(f); 

  // prepare the necessary memory space 
  allocMem();

  // and read the actual data
  for(i=0; i<dim; i++)
    for(j=0; j<dim; j++) 
      if((data[i][j]=fgetc(f))==EOF)
	fatal("File too short: ", name);
    
  // just to make sure the file is valid
  if (fgetc(f)!=EOF)
    fatal("File too long: ", name);
  fclose(f);
}

////////////////////////////////////////

void 
URLDEM::read()
{
  Timer *dt = new Timer;
  
  WebNode *node = new WebNode(name);
  dt->stop("URLDEM download time");
  

  char *buf = node->getBuffer();
  dim = ((buf[0]*256 + buf[1])*256 + buf[2])*256 + buf[3];

  buf = buf + 8;

  allocMem();

  int i, j;
  for(i=0; i<dim; i++) 
    for(j=0; j<dim; j++)
	data[i][dim-j-1]=buf[2*(i*dim+j)]*256.0+buf[2*(i*dim+j)+1];

  delete node->getBuffer();
}

////////////////////////////////////////

void 
URLCompressedDEM::read()
{
  Timer *dt = new Timer;
  
  WebNode *node = new WebNode(name);
  dt->stop("URL compressed DEM downlaod time");
  

  char *buf = node->getBuffer();

  dim = buf[0]*256 + buf[1];
  
  allocMem();
  
  buf = buf + 2;

  wv_t *wv = new_wv(dim);
  int streamLength = node->getBufSize();
  dt->reset();
  
  dec_elev_wv(wv,(unsigned char*)buf,streamLength,data);

  dt->stop("URL compressed DEM decompression time");
  
  for(int i=0; i<dim; i++)
    for(int j=0; j<dim; j++)
      data[i][j] = data[i][j]*MULT;
  
  
  delete node->getBuffer();
  delete_wv(&wv);
}

